import re

from . import mail

# import unidecode


def slugify(text):
    # text = unidecode.unidecode(text)
    return re.sub(r"[\W_]+", "_", text.lower())


def extract_uniques(elements, comp=lambda left, right: left == right):
    keep = []
    for e in elements:
        if not any(comp(e, k) for k in keep):
            keep.append(e)
    return keep



import json
import logging
import os

import requests

from .domain import Domains
from .record import Records

DEFAULT_HEADERS = {
    "Accept": "application/json",
    "Content-Type": "application/json",
}

DEFAULT_BASE_URL = "https://apiv2.mtgsy.net/api/v1"


def parse_response(response):
    res = response.json()
    error = res.get("error")
    if error:
        logging.debug(error)
        raise Exception(error.get("description", "Unknown error"))
    return res.get("data")


class BaseClient:
    def __init__(self, username=None, apikey=None, url=DEFAULT_BASE_URL) -> None:
        if not username:
            username = os.environ.get("CLOUDFLOOR_USERNAME", "").strip()
        if not username:
            raise Exception("username required")

        if not apikey:
            apikey = os.environ.get("CLOUDFLOOR_APIKEY", "").strip()
        if not apikey:
            raise Exception("username required")
        self._username = username
        self._apikey = apikey
        self._url = url.rstrip("/")

    def request(self, method, url, data=None, timeout=None):
        if not url.startswith("/"):
            raise Exception(
                f"url '{url}' is invalid: must be a path with a leading '/' "
            )
        if not data:
            data = {}
        data = {
            **data,
            "username": self._username,
            "apikey": self._apikey,
        }
        url = f"{self._url}{url}"
        res = requests.request(
            method,
            url,
            headers=DEFAULT_HEADERS,
            data=json.dumps(data),
            allow_redirects=True,
            timeout=timeout,
        )
        return parse_response(res)

    def get(self, url, data=None, timeout=None):
        return self.request("GET", url, data=data, timeout=timeout)

    def post(self, url, data=None, timeout=None):
        return self.request("POST", url, data=data, timeout=timeout)

    def patch(self, url, data=None, timeout=None):
        return self.request("PATCH", url, data=data, timeout=timeout)

    def delete(self, url, data=None, timeout=None):
        return self.request("DELETE", url, data=data, timeout=timeout)


class Client(BaseClient):
    def __init__(self, username=None, apikey=None, url=DEFAULT_BASE_URL) -> None:
        super().__init__(username=username, apikey=apikey, url=url)
        self.records = Records(self)
        self.domains = Domains(self)



# from dataclasses import dataclass, field
from typing import List

from pydantic import BaseModel, Extra, Field


class Domain(BaseModel):
    domainname: str

    id: str = None
    registered: int = None
    secondary: int = None
    primary: int = None
    group_ids: List[str] = Field(default_factory=list)

    class Config:
        populate_by_name = True
        extra = Extra.allow

    # def __eq__(self, __value: Record) -> bool:
    #     if not isinstance(__value, Record):
    #         return NotImplemented
    #     fields1 = self.model_dump(exclude_unset=True)
    #     fields2 = __value.model_dump(exclude_unset=True)
    #     try:
    #         return all(fields2[k] == v for k, v in fields1.items())
    #     except Exception:
    #         return False

    def is_same(self, right: "Domain") -> bool:
        """
        This method check the identity (e.g. same id if defined, or same name/name+value)
        """
        if not isinstance(right, Domain):
            return NotImplemented
        return self.domainname == self.right


class Domains:
    def __init__(self, client) -> None:
        self.client = client

    # def create(self, domain: str, record: Record, timeout=None):
    #     url = f"/dns/zone/{domain}/record"
    #     return self.client.post(
    #         url,
    #         data=record.model_dump(),
    #         timeout=timeout,
    #     )

    # def update(self, domain: str, record_id: str, record: Record, timeout=None):
    #     url = f"/dns/zone/{domain}/record/{record_id}"
    #     return self.client.patch(
    #         url,
    #         data=record.model_dump(exclude_unset=True),
    #         timeout=timeout,
    #     )

    # def delete(self, domain: str, record_id: str, timeout=None):
    #     url = f"/dns/zone/{domain}/record/{record_id}"
    #     return self.client.delete(
    #         url,
    #         timeout=timeout,
    #     )

    def list(self, timeout=None):
        url = "/dns/zone"
        res = self.client.get(
            url,
            timeout=timeout,
        )
        return [Domain(**d) for d in res]

    def get(self, domain: str, domain_id, timeout=None):
        res = self.client.list(
            domain,
            timeout=timeout,
        )
        return next((r for r in res if r.id == domain_id), None)

    def get_by_name(self, domain: str, domainname, timeout=None):
        res = self.client.list(
            domain,
            timeout=timeout,
        )
        return next((r for r in res if r.domainname == domainname), None)



# from dataclasses import dataclass, field
from typing import List

from pydantic import BaseModel, Extra, field


class Domain(BaseModel):
    domainname: str

    id: str = None
    organisation: str = None
    ownerfirstname: str = None
    ownerlastname: str = None
    ownercompanyname: str = None
    ownerstreetaddress: str = None
    ownercity: str = None
    ownerstate: str = None
    ownerpostalcode: str = None
    ownercountry: str = None
    ownerphone: str = None
    ownerfax: str = None
    owneremail: str = None
    adminlastname: str = None
    admincompanyname: str = None
    adminstreetaddress: str = None
    admincity: str = None
    adminstate: str = None
    adminpostalcode: str = None
    admincountry: str = None
    adminphone: str = None
    adminfax: str = None
    adminemail: str = None
    billfirstname: str = None
    billlastname: str = None
    billcompanyname: str = None
    billstreetaddress: str = None
    billcity: str = None
    billstate: str = None
    billpostalcode: str = None
    billcountry: str = None
    billphone: str = None
    billfax: str = None
    billemail: str = None
    techfirstname: str = None
    techlastname: str = None
    techcompanyname: str = None
    techstreetaddress: str = None
    techcity: str = None
    techstate: str = None
    techcountry: str = None
    techpostalcode: str = None
    techphone: str = None
    techfax: str = None
    techemail: str = None
    auto_renew: str = None
    reg_opt_out: str = None
    username: str = None
    editzone: str = None
    expires: str = None
    deleteonexpiry: str = None
    companyregno: str = None
    client_delete_prohibited_lock: str = None
    client_update_prohibited_lock: str = None
    client_transfer_prohibited_lock: str = None
    registeredhere: str = None
    nameservers: List[str] = field(default_factory=list)

    class Config:
        populate_by_name = True
        extra = Extra.allow

    # def __eq__(self, __value: Record) -> bool:
    #     if not isinstance(__value, Record):
    #         return NotImplemented
    #     fields1 = self.model_dump(exclude_unset=True)
    #     fields2 = __value.model_dump(exclude_unset=True)
    #     try:
    #         return all(fields2[k] == v for k, v in fields1.items())
    #     except Exception:
    #         return False

    def is_same(self, right: "Domain") -> bool:
        """
        This method check the identity (e.g. same id if defined, or same name/name+value)
        """
        if not isinstance(right, Domain):
            return NotImplemented
        return self.domainname == self.right


class Domains:
    def __init__(self, client) -> None:
        self.client = client

    # def create(self, domain: str, record: Record, timeout=None):
    #     url = f"/dns/zone/{domain}/record"
    #     return self.client.post(
    #         url,
    #         data=record.model_dump(),
    #         timeout=timeout,
    #     )

    # def update(self, domain: str, record_id: str, record: Record, timeout=None):
    #     url = f"/dns/zone/{domain}/record/{record_id}"
    #     return self.client.patch(
    #         url,
    #         data=record.model_dump(exclude_unset=True),
    #         timeout=timeout,
    #     )

    # def delete(self, domain: str, record_id: str, timeout=None):
    #     url = f"/dns/zone/{domain}/record/{record_id}"
    #     return self.client.delete(
    #         url,
    #         timeout=timeout,
    #     )

    def get(self, domain: str, timeout=None):
        url = f"/domain/{domain}"
        res = self.client.get(
            url,
            timeout=timeout,
        )
        return [Domain(**d) for d in res]

    # def get(self, domain: str, record_id, timeout=None):
    #     res = self.client.list(
    #         domain,
    #         timeout=timeout,
    #     )
    #     return next((r for r in res if r.id == record_id), None)



import click

# from nagra_network_misc_utils.logger import set_default_logger

# from cloudfloordns.utils import main

# This sets a convenient global handler for the logs
# set_default_logger()

# Display info logs (you can change the logging level)
# logging.getLogger().setLevel(logging.INFO)


# Keep it as the main entry point
@click.group()
def cli():
    pass


# Add here your subcommands if needed
# cli.add_command(main)

if __name__ == "__main__":
    cli()



# from dataclasses import dataclass, field
from typing import Literal

from pydantic import BaseModel, Extra

TYPES_VALUES = Literal[
    "A",
    "AAAA",
    "ALIAS" "CNAME",
    "HINFO" "MX",
    "NS",
    "PTR",
    "RP" "SRV",
    "CAA",
    "TXT",
    "REDIRECT://",
    # For comparison, the following are valid on Cloudflare
    # "SOA",
    # "DS",
    # "DNSKEY",
    # "LOC",
    # "NAPTR",
    # "SSHFP",
    # "SVCB",
    # "TSLA",
    # "URI",
    # "SPF",
]

FULL_COMPARISON = {
    "A",
    "AAAA",
    "ALIAS" "CNAME",
}

UNIQUE_BY_NAME = {"HINFO" "MX", "NS", "PTR", "RP" "SRV", "CAA", "TXT", "REDIRECT://"}


class Record(BaseModel):
    name: str
    type: str
    data: str
    id: str = None
    zone: str = None
    aux: str = "0"
    ttl: int = 3600
    active: Literal["Y", "N"] = "Y"
    # isfwd: str = "0"
    # cc: str = None
    # lbType: str = "0"

    class Config:
        populate_by_name = True
        extra = Extra.allow

    # def __eq__(self, __value: Record) -> bool:
    #     if not isinstance(__value, Record):
    #         return NotImplemented
    #     fields1 = self.model_dump(exclude_unset=True)
    #     fields2 = __value.model_dump(exclude_unset=True)
    #     try:
    #         return all(fields2[k] == v for k, v in fields1.items())
    #     except Exception:
    #         return False

    def is_same(self, right: "Record") -> bool:
        """
        This method check the identity (e.g. same id if defined, or same name/name+value)
        """
        if not isinstance(right, Record):
            return NotImplemented
        if self.id and right.id:
            return self.id == right.id
        if (self.name, self.type) != (right.name, right.type):
            return False
        if self.type in FULL_COMPARISON:
            return self.data == right.data
        return True

    @property
    def is_spf(self) -> bool:
        return all((not self.name, self.type == "TXT", "v=spf" in self.data.lower()))

    @property
    def is_dkim(self) -> bool:
        return all(
            (
                "._domainkey" in self.name,
                self.type == "TXT",
                "v=dkim" in self.data.lower(),
            )
        )

    @property
    def is_dmarc(self) -> bool:
        return all(
            ("_dmarc" in self.name, self.type == "TXT", "v=dmarc" in self.data.lower())
        )


class Records:
    def __init__(self, client) -> None:
        self.client = client

    def create(self, domain: str, record: Record, timeout=None):
        url = f"/dns/zone/{domain}/record"
        return self.client.post(
            url,
            data=record.model_dump(),
            timeout=timeout,
        )

    def update(self, domain: str, record_id: str, record: Record, timeout=None):
        url = f"/dns/zone/{domain}/record/{record_id}"
        return self.client.patch(
            url,
            data=record.model_dump(exclude_unset=True),
            timeout=timeout,
        )

    def delete(self, domain: str, record_id: str, timeout=None):
        url = f"/dns/zone/{domain}/record/{record_id}"
        return self.client.delete(
            url,
            timeout=timeout,
        )

    def list(self, domain: str, timeout=None):
        url = f"/dns/zone/{domain}/record"
        res = self.client.get(
            url,
            timeout=timeout,
        )
        return [Record(**d) for d in res]

    def get(self, domain: str, record_id, timeout=None):
        res = self.client.list(
            domain,
            timeout=timeout,
        )
        return next((r for r in res if r.id == record_id), None)



import logging

from cloudfloordns.record import Record

DEFAULT_SPF = Record(name="", type="TXT", data="""v=spf1 -all""")
DEFAULT_DKIM = Record(name="*._domainkey", type="TXT", data="""v=DKIM1; p=""")
DEFAULT_DMARC = Record(
    name="_dmarc",
    type="TXT",
    data="""v=DMARC1; p=reject; sp=reject; adkim=s; aspf=s;""",
)
DEFAULT_MX = Record(name="", type="MX", data="")


def create_default_mail_records(client, domain, override=False, logger=None):
    if logger is None:
        logger = logging
    records = client.records.list(domain)
    spf_records = [r for r in records if r.is_spf]
    dkim_records = [r for r in records if r.is_dkim]
    dmarc_records = [r for r in records if r.is_dmarc]
    mx_records = [r for r in records if r.type == "MX"]

    if not spf_records or override:
        logger.info("Creating default SPF record for 'domain'")
        client.records.create(domain, DEFAULT_SPF)
    if not dkim_records or override:
        logger.info("Creating default DKIM record for 'domain'")
        client.records.create(domain, DEFAULT_DKIM)
    if not dmarc_records or override:
        logger.info("Creating default DMARC record for 'domain'")
        client.records.create(domain, DEFAULT_DMARC)
    if not mx_records or override:
        logger.info("Creating default MX record for 'domain'")
        client.records.create(domain, DEFAULT_MX)
